/***********************************************************************************
Projektname: Conways Game of Life
Author: Markus Foitschik (Nidhoegger)
Hardware used: ATMega8, 10x10 LED Matrix
Rules for the Game:
A living cell dies with 1 or 0 neighbours, as of loneliness
A living cell survives with 2 or 3 neighbours
A living cell dies with 4 or more neighbours, as of overpopulation
A dead cell gets born with exaclty 3 neighbours
License:
You can edit and change the project, but you may not release my original work on
other places, you will aknowlegde that i do not gibe warranties if something goes
wrong, you will leave my copyright and URL to the original project here. And you
will acknowlegde that my english is under all pig and wont shout at me because of
this.
***********************************************************************************/

#include <avr/io.h>
#include <avr/interrupt.h>
#include <stdint.h>
#include <avr/signal.h>
#include <util/delay.h>
#include <stdbool.h>
#include <stdlib.h>

/*
Variablen:
leds: speichert den momentanen zustand der LEDs (logischerweise: true=an, false=aus)
leds_copy: in der timerroutine werden die neuen zustnde gepspeichert
leds_check, leds_check2: hier werden die 2 letzten zustnde gespeichert um eine endlosschleife oder ein Standbild festzustellen
kill_game: gibt die anweisung das Game of Life neu zu starten
kill_count: wird bentigt um vor dem neustart das aktuelle bild noch etwas anzuzeigen
count: zhlt die jeweiligen nachbarn, knnte man auch als lokale var machen, ich habs halt hier rein gemacht, damit nicht immer
       speicher allokiert werden muss (wei nicht wieviel zeit das auf einem C braucht)
x1,x2,y1,y2: Hilftvariablen, die die schleife erheblich verlangsamen, sind dazu da um grenzbergreifend zu agieren
counter: zhlt, wie oft ein overflow stattfand. ggf. erhhen, wenn ein hheres Quarz eingesetzt wird, momentan ist es auf ein
         8MhZ quarz abgestimmt.

*/
volatile bool leds[10][10];
volatile bool leds_copy[10][10];
volatile bool leds_check[10][10];
volatile bool leds_check2[10][10];
volatile bool kill_game;
volatile int kill_count;
volatile short count, x1, x2, y1, y2;
volatile int counter;

// Timer overflow
ISR(TIMER0_OVF_vect)
{
    // Nur bei jedem 10. overflow ausfhren
    counter++;
    if (counter >= 10)
    {
        counter = 0;
        // Nachbarn berechnen
        for (int x = 0; x<=9; x++)
        {
            for (int y = 0; y<=9; y++)
            {
                count = 0;
                x1 = x-1;
                x2 = x+1;
                y1 = y-1;
                y2 = y+1;
                if (x1 < 0)
                {
                    x1 = 9;
                }
                if (x2 > 9)
                {
                    x1 = 0;
                }
                if (y1 < 0)
                {
                    y1 = 9;
                }
                if (y2 > 9)
                {
                    y2 = 0;
                }

                if (leds[x1][y1])
                {
                    count++;
                }
                if (leds[x1][y])
                {
                    count++;
                }
                if (leds[x1][y2])
                {
                    count++;
                }

                if (leds[x][y1])
                {
                    count++;
                }
                if (leds[x][y2])
                {
                    count++;
                }

                if (leds[x2][y1])
                {
                    count++;
                }
                if (leds[x2][y])
                {
                    count++;
                }
                if (leds[x2][y2])
                {
                    count++;
                }
                // neuen zustand in leds_copy speichern
                if (leds[x][y])
                {
                    if (count <= 1)
                    {
                        leds_copy[x][y] = false;
                    }
                    else if ((count == 2) || (count == 3))
                    {
                        leds_copy[x][y] = true;
                    }
                    else if (count > 3)
                    {
                        leds_copy[x][y] = false;
                    }
                }
                else
                {
                    if (count == 3)
                    {
                        leds_copy[x][y] = true;
                    }
                }




            }
        }
        // leds_copy nach leds schreiben und gleichzeitig letzte 2 zustnde mit dem jetzigen vergleichen, wenn beide sich unterscheiden
        // kill auf false setzen (kill wird spter kill_game zugewiesen
        bool kill;
        kill = true;

        for (int i = 0; i <= 9; i++)
        {
            for (int j = 0; j <= 9; j++)
            {
                if ((leds_copy[i][j] != leds_check[i][j]) && (leds_copy[i][j] != leds_check2[i][j]))
                {
                    kill = false;
                }
                leds[i][j] = leds_copy[i][j];
                leds_check2[i][j] = leds_check[i][j];
                leds_check[i][j] = leds[i][j];
            }
        }
        counter=0;
        kill_game = kill;
    }
}


// Timer einstellungen
void timer (void)
{
    TCCR0 |= (1<<CS00)|(1<<CS02);
    TIMSK   = (1<<TOIE0);
    sei();                 // Vorteiler ist 1024
}

int main (void)
{
    // Ein- und Ausgnge setzen
    DDRB  = 0b00111111;
    DDRC  = 0b00111111;
    DDRD  = 0b11111111;
    // ALLE LEDS AN!
    PORTB = 0b00111111;
    PORTC = 0b00001111;
    PORTD = 0b00000000;

    // Alle LEDs in der Matrix ausschalten:
    for (int i = 0; i <= 9; i++)
    {
        for (int j = 0; j <= 9; j++)
        {
            leds[i][j] = false;
        }
    }
    // Da erster anlauf, muss feld initialisiert werden
    kill_game = true;
    kill_count = 999;
    // timer aktivieren
    timer();
    while (1)
    {
        // Damit man die letzte position eines spiels noch etwas betrachten kann, erst nach 1000 increments von kill_count das spiel neu
        // initialisieren
        if (kill_game)
        {
            kill_count++;
        }
        if ((kill_game) && (kill_count >= 1000))
        {
            // alle felder zufllig auffllen
            kill_game = false;
            for (int i = 0; i <= 9; i++)
            {
                for (int j = 0; j <= 9; j++)
                {
                    if (rand() / (RAND_MAX / 2 + 1) == 1)
                    {
                        leds_copy[i][j] = true;
                        leds_check[i][j] = false;
                    }
                    else
                    {
                        leds_copy[i][j] = false;
                        leds_check[i][j] = true;
                    }
                }
            }
            kill_count = 0;
        }
        // Hier kommt die ausgabe der anzeigen
        // hier habe ich ein seltsames phnomen mitbekommen, das ich relativ drastisch gelst habe
        // wenn man nur eine anzeigen macht mit der hiesigen schleife, ergibt sich ein helligkeitsverlauf von links nach rechts,
        // wobei die LEDs rechts auf der anzeige dunkler waren, also habe ich einfach durchlufe von allein seiten gemacht, je mit
        // inkrements und dekrements. Kann man sicherlich optimieren, bin ich auch grade dran.
        PORTC = 0b00110000;
        PORTB = 0b00000000;
        PORTD = 0b11111111;
        for (int x = 0; x<=9; x++)
        {
            PORTC = 0b00110000;
            PORTB = 0b00000000;
            PORTD = 0b11111111;
            for (int y = 0; y<= 9; y++)
            {
                if (leds[x][y])
                {
                    if (x <= 5)
                    {
                        PORTB |= (1 << x);
                    }
                    else
                    {
                        PORTC |= (1 << (x - 6));
                    }
                    if (y <= 7)
                    {
                        PORTD &= ~(1 << y);
                    }
                    else
                    {
                        PORTC &= ~(1 << (y-4));
                    }
                }
            }
        }
        for (int y = 0; y<=9; y++)
        {
            PORTC = 0b00110000;
            PORTB = 0b00000000;
            PORTD = 0b11111111;
            for (int x = 0; x<= 9; x++)
            {
                if (leds[x][y])
                {
                    if (x <= 5)
                    {
                        PORTB |= (1 << x);
                    }
                    else
                    {
                        PORTC |= (1 << (x - 6));
                    }
                    if (y <= 7)
                    {
                        PORTD &= ~(1 << y);
                    }
                    else
                    {
                        PORTC &= ~(1 << (y-4));
                    }
                }
            }
        }

        for (int x = 9; x>=0; x--)
        {
            PORTC = 0b00110000;
            PORTB = 0b00000000;
            PORTD = 0b11111111;
            for (int y = 9; y>=0; y--)
            {
                if (leds[x][y])
                {
                    if (x <= 5)
                    {
                        PORTB |= (1 << x);
                    }
                    else
                    {
                        PORTC |= (1 << (x - 6));
                    }
                    if (y <= 7)
                    {
                        PORTD &= ~(1 << y);
                    }
                    else
                    {
                        PORTC &= ~(1 << (y-4));
                    }
                }
            }
        }

        for (int y = 9; y>=0; y--)
        {
            PORTC = 0b00110000;
            PORTB = 0b00000000;
            PORTD = 0b11111111;
            for (int x = 9; x>=0; x--)
            {
                if (leds[x][y])
                {
                    if (x <= 5)
                    {
                        PORTB |= (1 << x);
                    }
                    else
                    {
                        PORTC |= (1 << (x - 6));
                    }
                    if (y <= 7)
                    {
                        PORTD &= ~(1 << y);
                    }
                    else
                    {
                        PORTC &= ~(1 << (y-4));
                    }
                }
            }
        }
    }
    return 0;
}


